#ifndef FIXPOINT_H
#define FIXPOINT_H 1

/*
  Macros to define a simple 3-operation fixed-point library.
  Not the best implementation (which uses 64-bit regs), but mostly works.
*/

/* bits after the decimal point */
#define FIXSHIFT 16
#define FIXMULT (1L << FIXSHIFT)

/* type to use for fixed point */
#define FIXED long

/* macros to get normal numbers (I = integer, F = floating-point) in and out
   of fixed-point */
#define FFIX(n) ((FIXED) ((n) * FIXMULT))
#define IFIX(n) (((FIXED) (n)) << FIXSHIFT)
#define UNIFIX(n) ((int) ((n) >> FIXSHIFT))
#define UNFFIX(n) (((float) (n)) / ((float) FIXMULT))

/* fixed-point operations (note no division) */
#define FIXADD(s1,s2) ((s1) + (s2))
#define FIXSUB(s1,s2) ((s1) - (s2))
#define FIXMUL(s1,s2) (((s1) * (s2)) >> FIXSHIFT)

/* define rational fixed-point fraction (a/b) */
#define FIXFRAC(a,b) (IFIX(a) / (FIXED) (b))

#endif
