/****************************************************************************
**                                                                         **
**  Title: KBD.C                                                           **
**                                                                         **
**  Description: Support code for interrupt-drive PC keyboard.             **
**                                                                         **
**  Author: Joseph R. Slayton                                              **
**          Electronic Warfare Associates                                  **
**          400 W. Reeves Ave.                                             **
**          Ridgecrest, CA 93555-2539                                      **
**                                                                         **
**  Revision History:                                                      **
**    09 Aug 1994  JRS  Created                                            **
**                                                                         **
****************************************************************************/

#define MODULE_KBD

#include <dos.h>

#include "kbd.h"

/* hold for old pointer */
void (__interrupt __far *KbdOldVec)(void);

static char AscTable[128] = {
  /*   0 */    0,   27,  '1',  '2',  '3',  '4',  '5',  '6',  '7',  '8',
  /*  10 */  '9',  '0',  '-',  '=',    8,    9,  'Q',  'W',  'E',  'R',
  /*  20 */  'T',  'Y',  'U',  'I',  'O',  'P',  '[',  ']',   13,    0,
  /*  30 */  'A',  'S',  'D',  'F',  'G',  'H',  'J',  'K',  'L',  ';',
  /*  40 */ '\'',  '`',    0, '\\',  'Z',  'X',  'C',  'V',  'B',  'N',
  /*  50 */  'M',  ',',  '.',  '/',    0,    0,    0,  ' ',    0,    0,
  /*  60 */    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  /*  70 */    0,  199,  200,  201,    0,  203,  204,  205,  '+',  207,
  /*  80 */  208,  209,  210,  211,    0,    0,    0,    0,    0,    0,
  /*  90 */    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  /* 100 */    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  /* 110 */    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  /* 120 */    0,    0,    0,    0,    0,    0,    0,    0,
};

/*-------------------------------------------------------------------------*/

void __interrupt __far KbdIntHandler(void)
/*
  Keyboard handler
*/
{

  register char x;         /* keyboard ack register contents */

  /* read scan code */
  KbdRecent = inp(0x60);

  /* acknowledge keyboard hardware */
  x = inp(0x61);
  outp(0x61, (x | 0x80));
  outp(0x61, x);

  /* issue EOI to interrupt controller */
  outp(0x20, 0x20);

  /* if it's an extend marker, dump it */
  if (KbdRecent == (char) 0xE0) {
    KbdRecent = 0;
  } /* if */

  else {

    /* if the code was a break code, clear the flag */
    if ((KbdRecent & 128) != 0) {
      KbdKeyDown[KbdRecent & 127] = 0;
      KbdKeyHit = 0;
      KbdASCII = 0;
    } /* if */

    /* make code, mark result */
    else {
      KbdKeyDown[KbdRecent & 127] = 1;
      KbdKeyHit = 1;
      KbdASCII = AscTable[KbdRecent & 127];
    } /* else */

  } /* else */

} /* KbdIntHandler */

/*-------------------------------------------------------------------------*/

void KbdInit(void)
/*
  uninstall our keyboard handler
*/
{

  /* set keyboard int to our handler */
  KbdOldVec = _dos_getvect(9);
  _dos_setvect(9, KbdIntHandler);

} /* KbdInit */

/*-------------------------------------------------------------------------*/

void KbdClose(void)
/*
  uninstall our keyboard handler
*/
{
  /* replace the original keyboard handler */
  _dos_setvect(9, KbdOldVec);
} /* KbdClose */

/*-------------------------------------------------------------------------*/

int kbhit(void)
/*
  return if a key is depressed
*/
{
  return KbdKeyHit;
} /* kbhit */

/*-------------------------------------------------------------------------*/

int getch(void)
/*
  return scan code value of keypress.  wait for keypress if one isn't down.
*/
{

  /* wait for keypress */
  while (!KbdKeyHit);

  /* return scan code */
  return KbdRecent & 127;

} /* getch */

/*-------------------------------------------------------------------------*/
