/****************************************************************************
**                                                                         **
**  Title: KBD.H                                                           **
**                                                                         **
**  Description: Definitions for interrupt-drive PC keyboard.              **
**                                                                         **
**  Author: Joseph R. Slayton                                              **
**          Electronic Warfare Associates                                  **
**          400 W. Reeves Ave.                                             **
**          Ridgecrest, CA 93555-2539                                      **
**                                                                         **
**  Revision History:                                                      **
**    09 Aug 1994  JRS  Created                                            **
**                                                                         **
****************************************************************************/

#if !defined(_KBD_H)
#define _KBD_H 1

/* scan codes for various keys.  The program sets up an array of flags in
   keys which corresponds to the keyboard scan codes.  Each flag is set
   to non-zero if the key is pressed and 0 if it isn't.  These values are
   the indices into the Keys array to test the flags. */

#define ESC_KEY             1
#define HOME_KEY            71
#define UP_ARROW_KEY        72
#define PGUP_KEY            73
#define MINUS_KEY           74
#define LEFT_ARROW_KEY      75
#define NUMBER_5_KEY        76
#define RIGHT_ARROW_KEY     77
#define PLUS_KEY            78
#define END_KEY             79
#define DOWN_ARROW_KEY      80
#define PGDN_KEY            81

#define SPACE_KEY           57

#define NUM_1_KEY           2
#define NUM_2_KEY           3
#define NUM_3_KEY           4
#define NUM_4_KEY           5
#define NUM_5_KEY           6
#define NUM_6_KEY           7
#define NUM_7_KEY           8
#define NUM_8_KEY           9
#define NUM_9_KEY           10
#define NUM_0_KEY           11

#define F1_KEY              59
#define F2_KEY              60
#define F3_KEY              61
#define F4_KEY              62
#define F5_KEY              63
#define F6_KEY              64
#define F7_KEY              65
#define F8_KEY              66
#define F9_KEY              67
#define F10_KEY             68


#define Q_KEY               16
#define W_KEY               17
#define E_KEY               18
#define R_KEY               19
#define T_KEY               20
#define Y_KEY               21
#define U_KEY               22
#define I_KEY               23
#define O_KEY               24
#define P_KEY               25

#define A_KEY               30
#define S_KEY               31
#define D_KEY               32
#define F_KEY               33
#define G_KEY               34
#define H_KEY               35
#define J_KEY               36
#define K_KEY               37
#define L_KEY               38

#define Z_KEY               44
#define X_KEY               45
#define C_KEY               46
#define V_KEY               47
#define B_KEY               48
#define N_KEY               49
#define M_KEY               50

/* prototypes for routines provided by the key module */
void KbdInit(void);
void KbdClose(void);
int kbhit(void);
int getch(void);

#if defined(MODULE_KBD)
volatile char KbdKeyDown[128] = { 0 }; /* is the specified key depressed? */
volatile char KbdKeyHit = 0;           /* is there currently a key depressed? */
volatile char KbdRecent = 0;           /* most recent key hit */
volatile char KbdASCII = 0;            /* ASCII value of most recent char (0 if none) */
#else
extern volatile char KbdKeyDown[128];
extern volatile char KbdKeyHit;
extern volatile char KbdRecent;
extern volatile char KbdASCII;
#endif

#endif

/*-------------------------------------------------------------------------*/

